//+------------------------------------------------------------------+
// #include <Bulaschev_Statistic.mqh>                                |
//+------------------------------------------------------------------+
#property copyright "Copyright 2010, Urain"
#property link      "https://login.mql5.com/ru/users/Urain"
//+------------------------------------------------------------------+
//| enumeration of fitness functions                                 |
//+------------------------------------------------------------------+
enum Enum_Efficiency
  {
   avg_enter_eff,
   stdev_enter_eff,
   avg_exit_eff,
   stdev_exit_eff,
   avg_trade_eff,
   stdev_trade_eff
  };
//+------------------------------------------------------------------+
//| structure of deal                                                |
//+------------------------------------------------------------------+
struct S_Stat_Deals
  {
public:
   ulong             DTicket;         // ticket of deal   
   ENUM_DEAL_TYPE    deals_type;      // type of deal
   ENUM_DEAL_ENTRY   deals_entry;     // direction of deal 
   double            deals_volume;    // volume of deal    
   double            deals_price;     // price of opening of deal   
   datetime          deals_date;      // time of opening of deal  

                     S_Stat_Deals(){};
                    ~S_Stat_Deals(){};
  };
//+------------------------------------------------------------------+
//| structure of trade                                               |
//+------------------------------------------------------------------+
struct S_Stat_Trades
  {
public:
   ulong             OTicket;         // ticket of opening deal 
   ulong             CTicket;         // ticket of closing deal     
   ENUM_DEAL_TYPE    trade_type;      // type of trade
   double            trade_volume;    // volume of trade
   double            max_price_trade; // Maximum price of trade
   double            min_price_trade; // Minimum price of trade
   double            enter_price;     // price of opening of trade
   datetime          enter_date;      // time of opening of trade 
   double            exit_price;      // price of closing of trade
   datetime          exit_date;       // time of closing of trade

   double            enter_efficiency;// effectiveness of entering
   double            exit_efficiency; // effectiveness of exiting
   double            trade_efficiency;// effectiveness of trade

                     S_Stat_Trades(){};
                    ~S_Stat_Trades(){};
  };
//+------------------------------------------------------------------+
//| class for transformation of deals into trades                    |
//+------------------------------------------------------------------+
class C_Pos
  {
public:
   S_Stat_Deals      m_deals_stats[];  // structure of deals
   S_Stat_Trades     m_trades_stats[]; // structure of trades
   long              pos_id;       // id of position
   string            symbol;       // symbol of position
   int               count_deals;  // number of deals
   int               count_trades; // number of trades
   int               trades_ends;  // number of closed trades
   int               DIGITS;// accuracy of minimum volume by the symbol of position  
                     C_Pos()
     {
      count_deals=0;
      count_trades=0;
      trades_ends=0;
     };
                    ~C_Pos(){};
   void              OnHistory();         // create the history of position
   void              OnHistoryTransform();// transform the history of position into the new system of interpretation
   void              efficiency();        // calculation of effectiveness by Bulachev's methodology

private:
   void              open_pos(int c);
   void              copy_pos(int x);
   void              close_pos(int i,int c);
   double            nd(double v){return(NormalizeDouble(v,DIGITS));};// normalization to minimum volume
   void              DigitMinLots(); // accuracy of minimum volume
   double            iHighest(string          symbol_name,// symbol name
                              ENUM_TIMEFRAMES  timeframe,  // period
                              datetime         start_time, // start date
                              datetime         stop_time   // end date
                              );
   double            iLowest(string          symbol_name,// symbol name
                             ENUM_TIMEFRAMES  timeframe,  // period
                             datetime         start_time, // start date
                             datetime         stop_time   // end date
                             );
  };
//+------------------------------------------------------------------+
//| accuracy of calculation of trade volumes                         |
//+------------------------------------------------------------------+
void C_Pos::DigitMinLots()
  {
   DIGITS=(int)-log10(SymbolInfoDouble(symbol,SYMBOL_VOLUME_STEP));
   if(DIGITS<0)DIGITS=0;
  }
//+------------------------------------------------------------------+
//| filling the structure of the history of deals                    |
//+------------------------------------------------------------------+
void C_Pos::OnHistory()
  {
   ArrayResize(m_deals_stats,count_deals);
   for(int i=0;i<count_deals;i++)
     {
      m_deals_stats[i].DTicket=HistoryDealGetTicket(i);
      m_deals_stats[i].deals_type=(ENUM_DEAL_TYPE)HistoryDealGetInteger(m_deals_stats[i].DTicket,DEAL_TYPE);   // type of deal
      m_deals_stats[i].deals_entry=(ENUM_DEAL_ENTRY)HistoryDealGetInteger(m_deals_stats[i].DTicket,DEAL_ENTRY);// direction of deal
      m_deals_stats[i].deals_volume=HistoryDealGetDouble(m_deals_stats[i].DTicket,DEAL_VOLUME);                // volume of deal
      m_deals_stats[i].deals_price=HistoryDealGetDouble(m_deals_stats[i].DTicket,DEAL_PRICE);                  // price of opening
      m_deals_stats[i].deals_date=(datetime)HistoryDealGetInteger(m_deals_stats[i].DTicket,DEAL_TIME);         // time of opening
     }
  };
//+------------------------------------------------------------------+
//| transformation of deals into trades (engine classes)             |
//+------------------------------------------------------------------+
void C_Pos::OnHistoryTransform()
  {
   DigitMinLots();// fill the value of DIGITS
   count_trades=0;trades_ends=0;
   ArrayResize(m_trades_stats,count_trades,count_deals);
   for(int c=0;c<count_deals;c++)
     {
      if(m_deals_stats[c].deals_entry==DEAL_ENTRY_IN)
        {
         open_pos(c);
        }
      else// else in
        {
         double POS=0;
         for(int i=trades_ends;i<count_trades;i++)POS+=m_trades_stats[i].trade_volume;

         if(m_deals_stats[c].deals_entry==DEAL_ENTRY_INOUT)
           {
            for(int i=trades_ends;i<count_trades;i++)close_pos(i,c);
            trades_ends=count_trades;
            open_pos(c);
            m_trades_stats[count_trades-1].trade_volume=m_deals_stats[c].deals_volume-POS;
           }
         else// else in/out
           {
            if(m_deals_stats[c].deals_entry==DEAL_ENTRY_OUT)
              {
               if(c==count_deals-1)// if it's the last deal
                 {
                  for(int i=trades_ends;i<count_trades;i++)close_pos(i,c);
                  trades_ends=count_trades-1;
                 }
               else// if it's not the last deal
                 {
                  double out_vol=nd(m_deals_stats[c].deals_volume);
                  while(nd(out_vol)>0)
                    {
                     if(nd(out_vol)>=nd(m_trades_stats[trades_ends].trade_volume))
                       {
                        close_pos(trades_ends,c);
                        out_vol-=nd(m_trades_stats[trades_ends].trade_volume);
                        trades_ends++;
                       }
                     else// if the remainder of closing position is less than the next trade
                       {
                        // move all trades one step forward
                        count_trades++;
                        ArrayResize(m_trades_stats,count_trades);
                        for(int x=count_trades-1;x>trades_ends;x--)copy_pos(x);
                        // open a copy on the difference of the current position and remainder
                        m_trades_stats[trades_ends+1].trade_volume=nd(m_trades_stats[trades_ends].trade_volume-out_vol);
                        // close the current trade with new volume, which is equal to remainder
                        close_pos(trades_ends,c);
                        m_trades_stats[trades_ends].trade_volume=nd(out_vol);
                        out_vol=0;
                        trades_ends++;
                       }
                    }// while(out_vol>0)
                 }// if it's not the last deal
              }// if out
           }// else in/out
        }// else in
     }
  };
//+------------------------------------------------------------------+
//| open new trade                                                   |
//+------------------------------------------------------------------+
void C_Pos::open_pos(int c)
  {
   ArrayResize(m_trades_stats,count_trades+1);
   m_trades_stats[count_trades].OTicket=m_deals_stats[c].DTicket;
   m_trades_stats[count_trades].trade_type=m_deals_stats[c].deals_type;
   m_trades_stats[count_trades].trade_volume=m_deals_stats[c].deals_volume;
   m_trades_stats[count_trades].enter_price=m_deals_stats[c].deals_price;
   m_trades_stats[count_trades].enter_date=m_deals_stats[c].deals_date;
   count_trades++;
  }
//+------------------------------------------------------------------+
//| copy trade                                                       |
//+------------------------------------------------------------------+
void C_Pos::copy_pos(int x)
  {
   m_trades_stats[x].OTicket=m_trades_stats[x-1].OTicket;
   m_trades_stats[x].trade_type=m_trades_stats[x-1].trade_type;
   m_trades_stats[x].trade_volume=m_trades_stats[x-1].trade_volume;
   m_trades_stats[x].enter_price=m_trades_stats[x-1].enter_price;
   m_trades_stats[x].enter_date=m_trades_stats[x-1].enter_date;
  }
//+------------------------------------------------------------------+
//| close trade                                                      |
//+------------------------------------------------------------------+
void C_Pos::close_pos(int i,int c)
  {
   m_trades_stats[i].CTicket=m_deals_stats[c].DTicket;
   m_trades_stats[i].exit_price=m_deals_stats[c].deals_price;
   m_trades_stats[i].exit_date=m_deals_stats[c].deals_date;
  }
//+------------------------------------------------------------------+
//| search for the maximum within the period start_time --> stop_time|
//+------------------------------------------------------------------+
double C_Pos::iHighest(string           symbol_name,// symbol name
                       ENUM_TIMEFRAMES  timeframe,  // period
                       datetime         start_time, // start date
                       datetime         stop_time   // end date
                       )
  {
   double  buf[];
   datetime  start_t=(start_time/60)*60;// normalization of time of opening 
   datetime  stop_t=(stop_time/60+1)*60;// normalization of time of closing  
   int period=CopyHigh(symbol_name,timeframe,start_t,stop_t,buf);
   double res=buf[0];
   for(int i=1;i<period;i++)
      res=fmax(res,buf[i]);
   return(res);
  }
//+------------------------------------------------------------------+
//|     start_time --> stop_time                |
//+------------------------------------------------------------------+
double C_Pos::iLowest(string           symbol_name,// symbol name
                      ENUM_TIMEFRAMES  timeframe,  // period
                      datetime         start_time, // start date
                      datetime         stop_time   // end date
                      )
  {
   double  buf[];
   datetime  start_t=(start_time/60)*60;// normalization of time of opening 
   datetime  stop_t=(stop_time/60+1)*60;// normalization of time of closing     
   int period=CopyLow(symbol_name,timeframe,start_t,stop_t,buf);
   double res=buf[0];
   for(int i=1;i<period;i++)
      res=fmin(res,buf[i]);
   return(res);
  }
//+------------------------------------------------------------------+
//| calculation of effectiveness                                     |
//+------------------------------------------------------------------+
void C_Pos::efficiency()
  {
   for(int i=0;i<count_trades;i++)
     {
      m_trades_stats[i].max_price_trade=iHighest(symbol,PERIOD_M1,m_trades_stats[i].enter_date,m_trades_stats[i].exit_date); // maximum price of trade
      m_trades_stats[i].min_price_trade=iLowest(symbol,PERIOD_M1,m_trades_stats[i].enter_date,m_trades_stats[i].exit_date);  // minimum price of trade
      double minimax=0;
      minimax=m_trades_stats[i].max_price_trade-m_trades_stats[i].min_price_trade;//    
      if(minimax!=0)minimax=1.0/minimax;
      if(m_trades_stats[i].trade_type==DEAL_TYPE_BUY)
        {
         //Effectiveness of entering a position
         m_trades_stats[i].enter_efficiency=(m_trades_stats[i].max_price_trade-m_trades_stats[i].enter_price)*minimax;
         //Effectiveness of exiting from a position
         m_trades_stats[i].exit_efficiency=(m_trades_stats[i].exit_price-m_trades_stats[i].min_price_trade)*minimax;
         //Effectiveness of a trade 
         m_trades_stats[i].trade_efficiency=(m_trades_stats[i].exit_price-m_trades_stats[i].enter_price)*minimax;
        }
      else
        {
         if(m_trades_stats[i].trade_type==DEAL_TYPE_SELL)
           {
            //Effectiveness of entering a position
            m_trades_stats[i].enter_efficiency=(m_trades_stats[i].enter_price-m_trades_stats[i].min_price_trade)*minimax;
            //Effectiveness of exiting from a position
            m_trades_stats[i].exit_efficiency=(m_trades_stats[i].max_price_trade-m_trades_stats[i].exit_price)*minimax;
            //Effectiveness of a trade 
            m_trades_stats[i].trade_efficiency=(m_trades_stats[i].enter_price-m_trades_stats[i].exit_price)*minimax;
           }
        }
     }
  }
//+------------------------------------------------------------------+
//| structure of effectiveness                                       |
//+------------------------------------------------------------------+
struct S_efficiency
  {
   double            enter_efficiency; // effectiveness of entering
   double            exit_efficiency;  // effectiveness of exiting
   double            trade_efficiency; // effectiveness of trade 
                     S_efficiency()
     {
      enter_efficiency=0;
      exit_efficiency=0;
      trade_efficiency=0;
     };
                    ~S_efficiency(){};
  };
//+------------------------------------------------------------------+
//| class of statistics of trading in whole                          |
//+------------------------------------------------------------------+
class C_PosStat:public C_Pos
  {
public:
   int               PosTotal;
   C_Pos             pos[];
   int               All_count_trades;
   S_efficiency      trade[];
   S_efficiency      avg;
   S_efficiency      stdev;
   string            exp_name;
                     C_PosStat(){PosTotal=0; All_count_trades=0;};
                    ~C_PosStat(){};
   void              OnPosStat();                         // engine classes
   void              OnTradesStat();                      // gathering information about trades in the common array

   void              WriteFileDeals(string folder="deals ");
   void              WriteFileTrades(string folder="trades ");
   void              WriteFileTrades_all(string folder="trades all ");
   void              WriteFileDealsHTML(string folder="deals ");
   void              WriteFileDealsHTML2(string folder="deals ");
   void              WriteFileTradesHTML(string folder="trades ");
   void              WriteFileTradesHTML2(string folder="trades ");
   string            enum_translit(ENUM_DEAL_ENTRY x,bool latin=true);// transformation of enumeration into string
   string            enum_translit(ENUM_DEAL_TYPE x,bool latin=true); // transformation of enumeration into string (overloaded)
private:

   S_efficiency      AVG(int count);                                        // arithmetical mean
   S_efficiency      STDEV(const S_efficiency &mo,int count);               // standard deviation
   S_efficiency      add(const S_efficiency &a,const S_efficiency &b);      //add
   S_efficiency      take(const S_efficiency &a,const S_efficiency &b);     //subtract
   S_efficiency      multiply(const S_efficiency &a,const S_efficiency &b); //multiply
   S_efficiency      divided(const S_efficiency &a,double b);               //divide
   S_efficiency      square_root(const S_efficiency &a);                    //square root
   string            Head_style(string title);
  };
//+------------------------------------------------------------------+
//| engine of the class (engine classes)                             |
//+------------------------------------------------------------------+
void C_PosStat::OnPosStat()
  {
   exp_name=MQL5InfoString(MQL5_PROGRAM_NAME);
   long  id_pos[];// auxiliary array for creating the history of positions
   if(HistorySelect(0,TimeCurrent()))
     {
      int HTD=HistoryDealsTotal();
      ArrayResize(id_pos,PosTotal,HTD);
      for(int i=0;i<HTD;i++)
        {
         ulong DTicket=(ulong)HistoryDealGetTicket(i);
         if((ENUM_DEAL_TYPE)HistoryDealGetInteger(DTicket,DEAL_TYPE)==DEAL_TYPE_BALANCE)
            continue;// if it's a balance deal, skip it
         long id=HistoryDealGetInteger(DTicket,DEAL_POSITION_ID);
         bool present=false; // initial state, position doesn't exist            
         for(int j=0;j<PosTotal;j++)
           { if(id==id_pos[j]){ present=true; break; } }// if there is a position already - break

         if(!present)// write id as a new position appears
           {
            PosTotal++;
            ArrayResize(id_pos,PosTotal);
            id_pos[PosTotal-1]=id;
           }
        }
     }
   ArrayResize(pos,PosTotal);

   for(int p=0;p<PosTotal;p++)
     {
      if(HistorySelectByPosition(id_pos[p]))
        {
         pos[p].pos_id=id_pos[p];
         pos[p].count_deals=HistoryDealsTotal();
         pos[p].symbol=HistoryDealGetString(HistoryDealGetTicket(0),DEAL_SYMBOL);
         pos[p].OnHistory();
         pos[p].OnHistoryTransform();
         pos[p].efficiency();
         All_count_trades+=pos[p].count_trades;
        }
     }
  }
//+------------------------------------------------------------------+
//| gathering information about into the common array                |
//+------------------------------------------------------------------+
void  C_PosStat::OnTradesStat()
  {
   int c=0;
   ArrayResize(trade,All_count_trades);
   for(int i=0;i<PosTotal;i++)
     {
      for(int j=0;j<pos[i].count_trades;j++)
        {
         trade[c].enter_efficiency=pos[i].m_trades_stats[j].enter_efficiency;
         trade[c].exit_efficiency =pos[i].m_trades_stats[j].exit_efficiency;
         trade[c].trade_efficiency=pos[i].m_trades_stats[j].trade_efficiency;
         c++;
        }
     }
   avg=AVG(All_count_trades);
   stdev=STDEV(avg,All_count_trades);
  }
//+------------------------------------------------------------------+
//| Arithmetical mean (operations with structure)                    |
//+------------------------------------------------------------------+
S_efficiency  C_PosStat::AVG(int count)
  {
   S_efficiency sum;
   for(int i=0;i<count;i++)
     {
      sum=add(sum,trade[i]);
     }
   return(divided(sum,(double)count));
  }
//+------------------------------------------------------------------+
//| Standard deviation (operations with structures)                  |
//+------------------------------------------------------------------+
S_efficiency  C_PosStat::STDEV(const S_efficiency &mo,int count)
  {
   S_efficiency sum;
   S_efficiency temp;
   for(int i=0;i<count;i++)
     {
      temp=take(trade[i],mo);
      temp=multiply(temp,temp);
      sum=add(sum,temp);
     }
   temp=divided(sum,(double)count);
   return(square_root(temp));
  }
//+------------------------------------------------------------------+
//| add (operations with structures)                                 |
//+------------------------------------------------------------------+
S_efficiency  C_PosStat::add(const S_efficiency &a,const S_efficiency &b)
  {
   S_efficiency  res;
   res.enter_efficiency=a.enter_efficiency+b.enter_efficiency;
   res.exit_efficiency =a.exit_efficiency+b.exit_efficiency;
   res.trade_efficiency=a.trade_efficiency+b.trade_efficiency;
   return(res);
  }
//+------------------------------------------------------------------+
//| subtract (operations with structures)                            |
//+------------------------------------------------------------------+
S_efficiency  C_PosStat::take(const S_efficiency &a,const S_efficiency &b)
  {
   S_efficiency  res;
   res.enter_efficiency=a.enter_efficiency-b.enter_efficiency;
   res.exit_efficiency =a.exit_efficiency-b.exit_efficiency;
   res.trade_efficiency=a.trade_efficiency-b.trade_efficiency;
   return(res);
  }
//+------------------------------------------------------------------+
//| multiply (operations with structures)                            |
//+------------------------------------------------------------------+
S_efficiency  C_PosStat::multiply(const S_efficiency &a,const S_efficiency &b)
  {
   S_efficiency  res;
   res.enter_efficiency=a.enter_efficiency*b.enter_efficiency;
   res.exit_efficiency =a.exit_efficiency*b.exit_efficiency;
   res.trade_efficiency=a.trade_efficiency*b.trade_efficiency;
   return(res);
  }
//+------------------------------------------------------------------+
//| divide (operations with structures)                              |
//+------------------------------------------------------------------+
S_efficiency  C_PosStat::divided(const S_efficiency &a,double b)
  {
   S_efficiency  res;
   double ib=0;if(b!=0.0)ib=1.0/b;
   res.enter_efficiency=a.enter_efficiency*ib;
   res.exit_efficiency =a.exit_efficiency*ib;
   res.trade_efficiency=a.trade_efficiency*ib;
   return(res);
  }
//+------------------------------------------------------------------+
//| square root (operations with structures)                         |
//+------------------------------------------------------------------+
S_efficiency  C_PosStat::square_root(const S_efficiency &a)
  {
   S_efficiency  res;
   res.enter_efficiency=sqrt(a.enter_efficiency);
   res.exit_efficiency =sqrt(a.exit_efficiency);
   res.trade_efficiency=sqrt(a.trade_efficiency);
   return(res);
  }
//+------------------------------------------------------------------+
//| csv report on deals                                              |
//+------------------------------------------------------------------+
void C_PosStat::WriteFileDeals(string folder="deals ")
  {
   string subfolder="OnHistory";
   int han=FileOpen(subfolder+"\\"+folder+exp_name+".csv",FILE_WRITE|FILE_CSV,";");
   if(han!=INVALID_HANDLE)
     {
      int c=0;
      for(int i=0;i<PosTotal;i++)
        {
         FileWrite(han,"pos["+(string)i+"]",
                   "id="+(string)pos[i].pos_id,pos[i].symbol,
                   "+-------------------------------------------+"
                   );
         for(int j=0;j<pos[i].count_deals;j++)
           {
            FileWrite(han,c,pos[i].m_deals_stats[j].DTicket,
                      enum_translit(pos[i].m_deals_stats[j].deals_type),
                      enum_translit(pos[i].m_deals_stats[j].deals_entry),
                      pos[i].m_deals_stats[j].deals_volume,
                      pos[i].m_deals_stats[j].deals_price,
                      pos[i].m_deals_stats[j].deals_date
                      );
            c++;
           }
        }
      FileClose(han);
     }
   else Print("File open failed"+subfolder+"\\"+folder+exp_name+".csv, error",GetLastError());
  }
//+------------------------------------------------------------------+
//| html report on deals                                             |
//+------------------------------------------------------------------+
void C_PosStat::WriteFileDealsHTML(string folder="deals ")
  {
   string subfolder="OnHistory";
   int han=FileOpen(subfolder+"\\"+folder+exp_name+".html",FILE_WRITE|FILE_ANSI," ");
   if(han!=INVALID_HANDLE)
     {
      FileWrite(han,"<html>",
                "<head>",
                "<title>Expert Deals Report</title>",
                "</head>",
                "<body bgcolor='#EFEFEF'>",
                "<center>",
                "<h2>Deals Report "+exp_name+"</h2>",
                "<table align='center' border='1' bgcolor='#FFFFFF' bordercolor='#7F7F7F' cellspacing='1' cellpadding='3'>");//7F7FFF
      FileWrite(han,
                "<tr>",
                "<th rowspan=1>#</th>",
                "<th rowspan=1>ticket</th>",
                "<th rowspan=1>type</th>",
                "<th rowspan=1>entry</th>",
                "<th rowspan=1>volume</th>",
                "<th rowspan=1>price</th>",
                "<th rowspan=1>time</th>",
                "</tr>",
                "<tr>","</tr>"
                );

      int c=0;
      for(int i=0;i<PosTotal;i++)
        {

         FileWrite(han,
                   "<tr>","</tr>",
                   "<tr align='center'>",
                   "<td bgcolor='#BFBFAF' rowspan=2 colspan=2>"+"<big>"+"pos["+(string)i+"]"+"</big>"+"</td>",
                   "<td bgcolor='#DFDFCF' rowspan=2 colspan=3>"+"<big>"+"id  "+(string)pos[i].pos_id+"</big>"+"</td>",
                   "<td bgcolor='#FFFFEF' rowspan=2 colspan=2>"+"<big>"+pos[i].symbol+"</big>"+"</td>",
                   "<td bgcolor='#FFFFFF'></td>",
                   "<td bgcolor='#FFFFFF'></td>",
                   "<td bgcolor='#FFFFFF'></td>",
                   "<td bgcolor='#FFFFFF'></td>",
                   "</tr>"
                   );
         for(int j=0;j<pos[i].count_deals;j++)
           {
            string tr="<tr bgcolor='#FFFFFF'>";
            if(c%2==0)tr="<tr bgcolor='#F2F2F2'>";
            FileWrite(han,
                      "<tr>","</tr>",
                      tr,
                      "<th rowspan=1>"+(string)c+"</th>",
                      "<th rowspan=1>"+(string)pos[i].m_deals_stats[j].DTicket+"</th>",
                      "<th rowspan=1>"+enum_translit(pos[i].m_deals_stats[j].deals_type)+"</th>",
                      "<th rowspan=1>"+enum_translit(pos[i].m_deals_stats[j].deals_entry)+"</th>",
                      "<th rowspan=1>"+DoubleToString(pos[i].m_deals_stats[j].deals_volume,1)+"</th>",
                      "<th rowspan=1>"+DoubleToString(pos[i].m_deals_stats[j].deals_price,5)+"</th>",
                      "<th rowspan=1>"+(string)pos[i].m_deals_stats[j].deals_date+"</th>",
                      "</tr>"
                      );
            c++;
           }
        }
      // forming the end of html file
      FileWrite(han,
                "</table><br><br>",
                "</center>",
                "</body>",
                "</html>");
      FileClose(han);
     }
   else Print("File open failed"+subfolder+"\\"+folder+exp_name+".html, error",GetLastError());
  }
//+------------------------------------------------------------------+
//| html report on deals                                             |
//+------------------------------------------------------------------+
string C_PosStat::Head_style(string title)
  {
   string res="";
   StringConcatenate(res,"<head>",
                     "<title>"+title+"</title>",
                     "<style type='text/css'>",
                     "<!--",
                     "@media screen {",
                     "td { font: 8pt  Tahoma,Arial; }",
                     "th { font: 10pt Tahoma,Arial; }",
                     "}",
                     "@media print {",
                     "td { font: 7pt Tahoma,Arial; }",
                     "th { font: 9pt Tahoma,Arial; }",
                     "}",
                     ".msdate { mso-number-format:'General Date'; }",
                     ".mspt   { mso-number-format:\#\,\#\#0\.00;  }",
                     "body {margin:1px;}",
                     "//-->",
                     "</style>",
                     "</head>"
                     );
   return(res);
  }
//+------------------------------------------------------------------+
//|  html report on deals 2                                          |
//+------------------------------------------------------------------+
void C_PosStat::WriteFileDealsHTML2(string folder="deals ")
  {
   string subfolder="OnHistory";
   int han=FileOpen(subfolder+"\\"+folder+exp_name+".html",FILE_WRITE|FILE_ANSI," ");
   if(han!=INVALID_HANDLE)
     {
      FileWrite(han,"<html>",Head_style("Expert Deals Report"));
      FileWrite(han,
                "<body bgcolor='DFDFDF'>",
                "<div align='center'>",
                "<table cellspacing='1' cellpadding='3' border='0'>",
                "<tr>",
                "<td nowrap style='height: 10px'></td>",
                "</tr>",
                "<tr align='center'>",
                "<th colspan='7' style='height: 25px'><div style='font: 10pt Tahoma'><b>Deals Report "+exp_name+"</b></div></th>",
                "</tr>",
                "<tr align='center' bgcolor='#BFFFBF'>",
                "<td nowrap style='height: 30px'><b>#</b></td>",
                "<td nowrap><b>Ticket</b></td>",
                "<td nowrap><b>Type</b></td>",
                "<td nowrap><b>Entry</b></td>",
                "<td nowrap><b>Volume</b></td>",
                "<td nowrap><b>Price</b></td>",
                "<td nowrap><b>Time</b></td>",
                "</tr>"
                );

      int c=0;
      for(int i=0;i<PosTotal;i++)
        {
         FileWrite(han,
                   "<tr>","</tr>",
                   "<tr align='center'>",//rowspan=2 +"<big>
                   "<td bgcolor='#BFBFAF' nowrap colspan=2>"+"pos["+(string)i+"]"+"</td>",
                   "<td bgcolor='#DFDFCF' nowrap colspan=3>"+"id  "+(string)pos[i].pos_id+"</td>",
                   "<td bgcolor='#FFFFEF' nowrap colspan=2>"+pos[i].symbol+"</td>",
                   "</tr>"
                   );
         for(int j=0;j<pos[i].count_deals;j++)
           {
            string tr="<tr bgcolor='#FFFFFF' align='center'>";
            if(c%2==0)tr="<tr bgcolor='#EFEFEF' align='center'>";
            FileWrite(han,
                      "<tr>","</tr>",
                      tr,
                      "<td nowrap>"+(string)c+"</td>",
                      "<td nowrap>"+(string)pos[i].m_deals_stats[j].DTicket+"</td>",
                      "<td nowrap>"+enum_translit(pos[i].m_deals_stats[j].deals_type)+"</td>",
                      "<td nowrap>"+enum_translit(pos[i].m_deals_stats[j].deals_entry)+"</td>",
                      "<td nowrap>"+DoubleToString(pos[i].m_deals_stats[j].deals_volume,1)+"</td>",
                      "<td nowrap>"+DoubleToString(pos[i].m_deals_stats[j].deals_price,5)+"</td>",
                      "<td nowrap>"+(string)pos[i].m_deals_stats[j].deals_date+"</td>",
                      "</tr>"
                      );
            c++;
           }
        }
      // forming the end of html file
      FileWrite(han,
                "</table>",
                "</div>",
                "</body>",
                "</html>");
      FileClose(han);
     }
   else Print("File open failed"+subfolder+"\\"+folder+exp_name+".html, error",GetLastError());
  }
//+------------------------------------------------------------------+
//| csv                                                 |
//+------------------------------------------------------------------+
void C_PosStat::WriteFileTrades(string folder="trades ")
  {
   string subfolder="OnHistory";
   int han=FileOpen(subfolder+"\\"+folder+exp_name+".csv",FILE_WRITE|FILE_CSV,";");
   if(han!=INVALID_HANDLE)
     {
      int c=0;
      for(int i=0;i<PosTotal;i++)
        {
         FileWrite(han,"pos["+(string)i+"]",
                   "id="+(string)pos[i].pos_id,pos[i].symbol,
                   "+---------------------------------------------------------------------------------------------------+"
                   );
         for(int j=0;j<pos[i].count_trades;j++)
           {
            FileWrite(han,c,pos[i].m_trades_stats[j].OTicket,pos[i].m_trades_stats[j].CTicket,
                      enum_translit(pos[i].m_trades_stats[j].trade_type),
                      pos[i].m_trades_stats[j].trade_volume,
                      pos[i].m_trades_stats[j].enter_price,
                      pos[i].m_trades_stats[j].enter_date,
                      pos[i].m_trades_stats[j].exit_price,
                      pos[i].m_trades_stats[j].exit_date,
                      pos[i].m_trades_stats[j].max_price_trade,
                      pos[i].m_trades_stats[j].min_price_trade,
                      pos[i].m_trades_stats[j].enter_efficiency,
                      pos[i].m_trades_stats[j].exit_efficiency,
                      pos[i].m_trades_stats[j].trade_efficiency
                      );
            c++;
           }
        }
      FileClose(han);
     }
   else Print("File open failed"+subfolder+"\\"+folder+".txt, error",GetLastError());
  }
//+------------------------------------------------------------------+
//| html report on trades                                            |
//+------------------------------------------------------------------+
void C_PosStat::WriteFileTradesHTML(string folder="trades ")
  {
   string subfolder="OnHistory";
   int han=FileOpen(subfolder+"\\"+folder+exp_name+".html",FILE_WRITE|FILE_ANSI," ");
   if(han!=INVALID_HANDLE)
     {
      FileWrite(han,"<html>",
                "<head>",
                "<title>Expert Trades Report</title>",
                "</head>",
                "<body bgcolor='#EFEFEF'>",
                "<center>",
                "<h2>Trades Report "+exp_name+"</h2>",
                "<table align='center' border='1' bgcolor='#FFFFFF' bordercolor='#7F7F7F' cellspacing='1' cellpadding='3'>");//7F7FFF
      FileWrite(han,
                "<tr>",
                "<th rowspan=2 >#</th>",
                "<th colspan=2 >Ticket</th>",
                "<th rowspan=2 >type</th>",
                "<th rowspan=2 >volume</th>",
                "<th colspan=2 >Open</th>",
                "<th colspan=2 >Close</th>",
                "<th colspan=2 >Price</th>",
                "<th rowspan=2 >enter efficiency</th>",
                "<th rowspan=2 >exit efficiency</th>",
                "<th rowspan=2 >trade efficiency</th>",
                "</tr>",
                "<tr>",
                "<th rowspan=1>open</th>",
                "<th rowspan=1>close</th>",
                "<th rowspan=1>price</th>",
                "<th rowspan=1>time</th>",
                "<th rowspan=1>price</th>",
                "<th rowspan=1>time</th>",
                "<th rowspan=1>max</th>",
                "<th rowspan=1>min</th>",
                "</tr>"
                );

      int c=0;
      for(int i=0;i<PosTotal;i++)
        {

         FileWrite(han,
                   "<tr>","</tr>",
                   "<tr align='center'>",
                   "<td bgcolor='#BFBFAF' rowspan=1 colspan=5>"+"<big>"+"pos["+(string)i+"]"+"</big>"+"</td>",
                   "<td bgcolor='#DFDFCF' rowspan=1 colspan=4>"+"<big>"+"id  "+(string)pos[i].pos_id+"</big>"+"</td>",
                   "<td bgcolor='#FFFFEF' rowspan=1 colspan=5>"+"<big>"+pos[i].symbol+"</big>"+"</td>",
                   "<td bgcolor='#FFFFFF'></td>",
                   "<td bgcolor='#FFFFFF'></td>",
                   "<td bgcolor='#FFFFFF'></td>",
                   "<td bgcolor='#FFFFFF'></td>",
                   "</tr>"
                   );
         int dig=(int)SymbolInfoInteger(pos[i].symbol,SYMBOL_DIGITS);
         for(int j=0;j<pos[i].count_trades;j++)
           {
            string tr="<tr bgcolor='#FFFFFF'>";
            if(c%2==0)tr="<tr bgcolor='#F2F2F2'>";
            FileWrite(han,
                      "<tr>","</tr>",
                      tr,
                      "<th rowspan=1>"+(string)c+"</th>",
                      "<th rowspan=1>"+(string)pos[i].m_trades_stats[j].OTicket+"</th>",
                      "<th rowspan=1>"+(string)pos[i].m_trades_stats[j].CTicket+"</th>",
                      "<th rowspan=1>"+enum_translit(pos[i].m_trades_stats[j].trade_type)+"</th>",
                      "<th rowspan=1>"+DoubleToString(pos[i].m_trades_stats[j].trade_volume,1)+"</th>",

                      "<th rowspan=1>"+DoubleToString(pos[i].m_trades_stats[j].enter_price,dig)+"</th>",
                      "<th rowspan=1>"+(string)pos[i].m_trades_stats[j].enter_date+"</th>",
                      "<th rowspan=1>"+DoubleToString(pos[i].m_trades_stats[j].exit_price,dig)+"</th>",
                      "<th rowspan=1>"+(string)pos[i].m_trades_stats[j].exit_date+"</th>",

                      "<th rowspan=1>"+DoubleToString(pos[i].m_trades_stats[j].max_price_trade,dig)+"</th>",
                      "<th rowspan=1>"+DoubleToString(pos[i].m_trades_stats[j].min_price_trade,dig)+"</th>",
                      "<th rowspan=1>"+DoubleToString(pos[i].m_trades_stats[j].enter_efficiency,5)+"</th>",
                      "<th rowspan=1>"+DoubleToString(pos[i].m_trades_stats[j].exit_efficiency,5)+"</th>",
                      "<th rowspan=1>"+DoubleToString(pos[i].m_trades_stats[j].trade_efficiency,5)+"</th>",
                      "</tr>"
                      );
            c++;
           }
        }
      // forming the end of html file
      FileWrite(han,
                "</table><br><br>",
                "</center>",
                "</body>",
                "</html>");
      FileClose(han);
     }
   else Print("File open failed"+subfolder+"\\"+folder+exp_name+".html, error",GetLastError());
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void C_PosStat::WriteFileTradesHTML2(string folder="trades ")
  {
   string subfolder="OnHistory";
   int han=FileOpen(subfolder+"\\"+folder+exp_name+".html",FILE_WRITE|FILE_ANSI," ");
   if(han!=INVALID_HANDLE)
     {
      FileWrite(han,"<html>",Head_style("Expert Trades Report"));
      FileWrite(han,
                "<body bgcolor='#DFDFDF'>",
                "<div align='center'>",
                "<table cellspacing='1' cellpadding='3' border='0'>",
                "<tr>",
                "<td nowrap style='height: 10px'></td>",
                "</tr>",
                "<tr align='center'>",
                "<th colspan='14' style='height: 25px'><div style='font: 10pt Tahoma'><b>Trades Report "+exp_name+"</b></div></th>",
                "</tr>",
                "<tr bgcolor='#BFFFBF'>",
                "<th rowspan=2 > # </th>",
                "<th colspan=2 > Ticket </th>",
                "<th rowspan=2 > type </th>",
                "<th rowspan=2 > volume </th>",
                "<th colspan=2 > Open </th>",
                "<th colspan=2 > Close </th>",
                "<th colspan=2 > Price </th>",
                "<th colspan=3 > Efficiency </th>",
                "</tr>",
                "<tr bgcolor='#BFFFBF'>",
                "<th rowspan=1> open </th>",
                "<th rowspan=1> close </th>",
                "<th rowspan=1> price </th>",
                "<th rowspan=1> time </th>",
                "<th rowspan=1> price </th>",
                "<th rowspan=1> time </th>",
                "<th rowspan=1> max </th>",
                "<th rowspan=1> min </th>",
                "<th rowspan=1> enter </th>",
                "<th rowspan=1> exit </th>",
                "<th rowspan=1> trade </th>",

                "</tr>"
                );

      int c=0;
      for(int i=0;i<PosTotal;i++)
        {

         FileWrite(han,
                   "<tr>","</tr>",
                   "<tr align='center'>",
                   "<td bgcolor='#BFBFAF' nowrap colspan=5>"+"pos["+(string)i+"]"+"</td>",
                   "<td bgcolor='#DFDFCF' nowrap colspan=4>"+"id  "+(string)pos[i].pos_id+"</td>",
                   "<td bgcolor='#FFFFEF' nowrap colspan=5>"+pos[i].symbol+"</td>",
                   "</tr>"
                   );
         int dig=(int)SymbolInfoInteger(pos[i].symbol,SYMBOL_DIGITS);
         for(int j=0;j<pos[i].count_trades;j++)
           {
            string tr="<tr bgcolor='#FFFFFF' align='center'>";
            if(c%2==0)tr="<tr bgcolor='#EFEFEF' align='center'>";
            FileWrite(han,
                      "<tr>","</tr>",
                      tr,
                      "<td nowrap >"+(string)c+"</td>",
                      "<td nowrap >"+(string)pos[i].m_trades_stats[j].OTicket+"</td>",
                      "<td nowrap >"+(string)pos[i].m_trades_stats[j].CTicket+"</td>",
                      "<td nowrap >"+enum_translit(pos[i].m_trades_stats[j].trade_type)+"</td>",
                      "<td nowrap >"+DoubleToString(pos[i].m_trades_stats[j].trade_volume,1)+"</td>",

                      "<td nowrap >"+DoubleToString(pos[i].m_trades_stats[j].enter_price,dig)+"</td>",
                      "<td nowrap >"+(string)pos[i].m_trades_stats[j].enter_date+"</td>",
                      "<td nowrap >"+DoubleToString(pos[i].m_trades_stats[j].exit_price,dig)+"</td>",
                      "<td nowrap >"+(string)pos[i].m_trades_stats[j].exit_date+"</td>",

                      "<td nowrap >"+DoubleToString(pos[i].m_trades_stats[j].max_price_trade,dig)+"</td>",
                      "<td nowrap >"+DoubleToString(pos[i].m_trades_stats[j].min_price_trade,dig)+"</td>",
                      "<td nowrap >"+DoubleToString(pos[i].m_trades_stats[j].enter_efficiency,5)+"</td>",
                      "<td nowrap >"+DoubleToString(pos[i].m_trades_stats[j].exit_efficiency,5)+"</td>",
                      "<td nowrap >"+DoubleToString(pos[i].m_trades_stats[j].trade_efficiency,5)+"</td>",
                      "</tr>"
                      );
            c++;
           }
        }
      // forming the end of html file
      FileWrite(han,
                "</table>",
                "<tr>","</tr>",
                "</div>",

                "<div align='center'>",
                "<table cellspacing='1' cellpadding='3' border='0'>",
                "<tr>",
                "<td nowrap style='height: 10px'></td>",
                "</tr>",
                "<tr align='center'>",
                "<th colspan='14' style='height: 25px'><div style='font: 10pt Tahoma'><b>Effectiveness Report</b></div></th>",
                "</tr>",
                //--------------
                "<tr>","</tr>",
                "<tr bgcolor='#BFFFBF'>",
                "<th rowspan=2 align='left'>Fitness Func</th>",
                "<th rowspan=2 >Average Value</th>",
                "<th rowspan=2 >Standard Deviation</th>",
                "</tr>",
                //--------------
                "<tr>","</tr>",
                "<tr bgcolor='#FFFFFF'>",
                "<th rowspan=2 bgcolor='#BFFFBF' align='left' >Enter</th>",
                "<th rowspan=2 >"+DoubleToString(avg.enter_efficiency,2)+"</th>",
                "<th rowspan=2 >"+DoubleToString(stdev.enter_efficiency,2)+"</th>",
                "</tr>",
                //--------------
                "<tr>","</tr>",
                "<tr bgcolor='#FFFFFF'>",
                "<th rowspan=2 bgcolor='#BFFFBF' align='left' >Exit</th>",
                "<th rowspan=2 >"+DoubleToString(avg.exit_efficiency,2)+"</th>",
                "<th rowspan=2 >"+DoubleToString(stdev.exit_efficiency,2)+"</th>",
                "</tr>",
                //--------------
                "<tr>","</tr>",
                "<tr bgcolor='#FFFFFF'>",
                "<th rowspan=2 bgcolor='#BFFFBF' align='left' >Trades</th>",
                "<th rowspan=2 >"+DoubleToString(avg.trade_efficiency,2)+"</th>",
                "<th rowspan=2 >"+DoubleToString(stdev.trade_efficiency,2)+"</th>",
                "</tr>",
                //--------------
                "</table>",
                "</div>",

                //--------------    
                "<tr>","</tr>",
                "<tr>","</tr>",
                //--------------

                "</body>",
                "</html>");
      FileClose(han);
     }
   else Print("File open failed"+subfolder+"\\"+folder+exp_name+".html, error",GetLastError());
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void C_PosStat::WriteFileTrades_all(string folder="trades all ")
  {
   string subfolder="OnHistory";
   int han=FileOpen(subfolder+"\\"+folder+exp_name+".csv",FILE_WRITE|FILE_CSV,";");
   if(han!=INVALID_HANDLE)
     {
      int c=0;
      for(int i=0;i<All_count_trades;i++)
        {
         FileWrite(han,
                   trade[i].enter_efficiency,
                   trade[i].exit_efficiency,
                   trade[i].trade_efficiency
                   );

        }
      FileClose(han);
     }
   else Print("File open failed"+subfolder+"\\"+folder+exp_name+".txt, error",GetLastError());
   Print("end");
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
string C_PosStat::enum_translit(ENUM_DEAL_ENTRY x,bool latin=true)
  {
   if(latin)
     {
      switch(x)
        {
         case DEAL_ENTRY_IN:   return("in");     break;
         case DEAL_ENTRY_OUT:  return("out");    break;
         case DEAL_ENTRY_INOUT:return("in/out"); break;
         case DEAL_ENTRY_STATE:return("state");  break;
         default:return("error");                break;
        }
     }
   else
     {
      switch(x)
        {
         case DEAL_ENTRY_IN:   return("Enter market");              break;
         case DEAL_ENTRY_OUT:  return("Exit from market");            break;
         case DEAL_ENTRY_INOUT:return("Reverse");                  break;
         case DEAL_ENTRY_STATE:return("Sign of status entry");  break;
         default:return("error");                                  break;
        }
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
string C_PosStat::enum_translit(ENUM_DEAL_TYPE x,bool latin=true)
  {
   if(latin)
     {
      switch(x)
        {
         case DEAL_TYPE_BUY:        return("buy");        break;
         case DEAL_TYPE_SELL:       return("sell");       break;
         case DEAL_TYPE_BALANCE:    return("balance");    break;
         case DEAL_TYPE_CREDIT:     return("credit");     break;
         case DEAL_TYPE_CHARGE:     return("charge");     break;
         case DEAL_TYPE_CORRECTION: return("correction"); break;
         default:return("error");                         break;
        }
     }
   else
     {
      switch(x)
        {
         case DEAL_TYPE_BUY:        return("Buy");               break;
         case DEAL_TYPE_SELL:       return("Sell");               break;
         case DEAL_TYPE_BALANCE:    return("Balance operation");    break;
         case DEAL_TYPE_CREDIT:     return("Charging of credit");    break;
         case DEAL_TYPE_CHARGE:     return("Additional charges");  break;
         case DEAL_TYPE_CORRECTION: return("Correction"); break;
         default:return("error");                                   break;
        }
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
